/* This file was generated by upb_generator from the input file:
 *
 *     xds/type/matcher/v3/range.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#include <stddef.h>
#include "upb/generated_code_support.h"
#include "xds/type/matcher/v3/range.upb_minitable.h"
#include "xds/type/v3/range.upb_minitable.h"
#include "xds/type/matcher/v3/matcher.upb_minitable.h"
#include "validate/validate.upb_minitable.h"

// Must be last.
#include "upb/port/def.inc"

extern const struct upb_MiniTable UPB_PRIVATE(_kUpb_MiniTable_StaticallyTreeShaken);
static const upb_MiniTableSubInternal xds_type_matcher_v3_Int64RangeMatcher__submsgs[1] = {
  {.UPB_PRIVATE(submsg) = &xds__type__matcher__v3__Int64RangeMatcher__RangeMatcher_msg_init_ptr},
};

static const upb_MiniTableField xds_type_matcher_v3_Int64RangeMatcher__fields[1] = {
  {1, 8, 0, 0, 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable xds__type__matcher__v3__Int64RangeMatcher_msg_init = {
  &xds_type_matcher_v3_Int64RangeMatcher__submsgs[0],
  &xds_type_matcher_v3_Int64RangeMatcher__fields[0],
  16, 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "xds.type.matcher.v3.Int64RangeMatcher",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f00000a, &upb_prm_1bt_max64b},
  })
};

const upb_MiniTable* xds__type__matcher__v3__Int64RangeMatcher_msg_init_ptr = &xds__type__matcher__v3__Int64RangeMatcher_msg_init;
static const upb_MiniTableSubInternal xds_type_matcher_v3_Int64RangeMatcher_RangeMatcher__submsgs[2] = {
  {.UPB_PRIVATE(submsg) = &xds__type__v3__Int64Range_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &xds__type__matcher__v3__Matcher__OnMatch_msg_init_ptr},
};

static const upb_MiniTableField xds_type_matcher_v3_Int64RangeMatcher_RangeMatcher__fields[2] = {
  {1, UPB_SIZE(12, 16), 0, 0, 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(16, 24), 64, 1, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable xds__type__matcher__v3__Int64RangeMatcher__RangeMatcher_msg_init = {
  &xds_type_matcher_v3_Int64RangeMatcher_RangeMatcher__submsgs[0],
  &xds_type_matcher_v3_Int64RangeMatcher_RangeMatcher__fields[0],
  UPB_SIZE(24, 32), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "xds.type.matcher.v3.Int64RangeMatcher.RangeMatcher",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_prm_1bt_maxmaxb},
  })
};

const upb_MiniTable* xds__type__matcher__v3__Int64RangeMatcher__RangeMatcher_msg_init_ptr = &xds__type__matcher__v3__Int64RangeMatcher__RangeMatcher_msg_init;
static const upb_MiniTableSubInternal xds_type_matcher_v3_Int32RangeMatcher__submsgs[1] = {
  {.UPB_PRIVATE(submsg) = &xds__type__matcher__v3__Int32RangeMatcher__RangeMatcher_msg_init_ptr},
};

static const upb_MiniTableField xds_type_matcher_v3_Int32RangeMatcher__fields[1] = {
  {1, 8, 0, 0, 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable xds__type__matcher__v3__Int32RangeMatcher_msg_init = {
  &xds_type_matcher_v3_Int32RangeMatcher__submsgs[0],
  &xds_type_matcher_v3_Int32RangeMatcher__fields[0],
  16, 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "xds.type.matcher.v3.Int32RangeMatcher",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f00000a, &upb_prm_1bt_max64b},
  })
};

const upb_MiniTable* xds__type__matcher__v3__Int32RangeMatcher_msg_init_ptr = &xds__type__matcher__v3__Int32RangeMatcher_msg_init;
static const upb_MiniTableSubInternal xds_type_matcher_v3_Int32RangeMatcher_RangeMatcher__submsgs[2] = {
  {.UPB_PRIVATE(submsg) = &xds__type__v3__Int32Range_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &xds__type__matcher__v3__Matcher__OnMatch_msg_init_ptr},
};

static const upb_MiniTableField xds_type_matcher_v3_Int32RangeMatcher_RangeMatcher__fields[2] = {
  {1, UPB_SIZE(12, 16), 0, 0, 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(16, 24), 64, 1, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable xds__type__matcher__v3__Int32RangeMatcher__RangeMatcher_msg_init = {
  &xds_type_matcher_v3_Int32RangeMatcher_RangeMatcher__submsgs[0],
  &xds_type_matcher_v3_Int32RangeMatcher_RangeMatcher__fields[0],
  UPB_SIZE(24, 32), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "xds.type.matcher.v3.Int32RangeMatcher.RangeMatcher",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_prm_1bt_maxmaxb},
  })
};

const upb_MiniTable* xds__type__matcher__v3__Int32RangeMatcher__RangeMatcher_msg_init_ptr = &xds__type__matcher__v3__Int32RangeMatcher__RangeMatcher_msg_init;
static const upb_MiniTableSubInternal xds_type_matcher_v3_DoubleRangeMatcher__submsgs[1] = {
  {.UPB_PRIVATE(submsg) = &xds__type__matcher__v3__DoubleRangeMatcher__RangeMatcher_msg_init_ptr},
};

static const upb_MiniTableField xds_type_matcher_v3_DoubleRangeMatcher__fields[1] = {
  {1, 8, 0, 0, 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable xds__type__matcher__v3__DoubleRangeMatcher_msg_init = {
  &xds_type_matcher_v3_DoubleRangeMatcher__submsgs[0],
  &xds_type_matcher_v3_DoubleRangeMatcher__fields[0],
  16, 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "xds.type.matcher.v3.DoubleRangeMatcher",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f00000a, &upb_prm_1bt_max64b},
  })
};

const upb_MiniTable* xds__type__matcher__v3__DoubleRangeMatcher_msg_init_ptr = &xds__type__matcher__v3__DoubleRangeMatcher_msg_init;
static const upb_MiniTableSubInternal xds_type_matcher_v3_DoubleRangeMatcher_RangeMatcher__submsgs[2] = {
  {.UPB_PRIVATE(submsg) = &xds__type__v3__DoubleRange_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &xds__type__matcher__v3__Matcher__OnMatch_msg_init_ptr},
};

static const upb_MiniTableField xds_type_matcher_v3_DoubleRangeMatcher_RangeMatcher__fields[2] = {
  {1, UPB_SIZE(12, 16), 0, 0, 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(16, 24), 64, 1, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable xds__type__matcher__v3__DoubleRangeMatcher__RangeMatcher_msg_init = {
  &xds_type_matcher_v3_DoubleRangeMatcher_RangeMatcher__submsgs[0],
  &xds_type_matcher_v3_DoubleRangeMatcher_RangeMatcher__fields[0],
  UPB_SIZE(24, 32), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "xds.type.matcher.v3.DoubleRangeMatcher.RangeMatcher",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_prm_1bt_maxmaxb},
  })
};

const upb_MiniTable* xds__type__matcher__v3__DoubleRangeMatcher__RangeMatcher_msg_init_ptr = &xds__type__matcher__v3__DoubleRangeMatcher__RangeMatcher_msg_init;
static const upb_MiniTable *messages_layout[6] = {
  &xds__type__matcher__v3__Int64RangeMatcher_msg_init,
  &xds__type__matcher__v3__Int64RangeMatcher__RangeMatcher_msg_init,
  &xds__type__matcher__v3__Int32RangeMatcher_msg_init,
  &xds__type__matcher__v3__Int32RangeMatcher__RangeMatcher_msg_init,
  &xds__type__matcher__v3__DoubleRangeMatcher_msg_init,
  &xds__type__matcher__v3__DoubleRangeMatcher__RangeMatcher_msg_init,
};

const upb_MiniTableFile xds_type_matcher_v3_range_proto_upb_file_layout = {
  messages_layout,
  NULL,
  NULL,
  6,
  0,
  0,
};

#include "upb/port/undef.inc"

