/* This file was generated by upb_generator from the input file:
 *
 *     src/proto/grpc/channelz/v2/property_list.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#ifndef SRC_PROTO_GRPC_CHANNELZ_V2_PROPERTY_LIST_PROTO_UPB_H__UPB_MINITABLE_H_
#define SRC_PROTO_GRPC_CHANNELZ_V2_PROPERTY_LIST_PROTO_UPB_H__UPB_MINITABLE_H_

#include "upb/generated_code_support.h"

// Must be last.
#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern const upb_MiniTable grpc__channelz__v2__PropertyList_msg_init;
extern const upb_MiniTable* grpc__channelz__v2__PropertyList_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v2__PropertyList__Element_msg_init;
extern const upb_MiniTable* grpc__channelz__v2__PropertyList__Element_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v2__PropertyGrid_msg_init;
extern const upb_MiniTable* grpc__channelz__v2__PropertyGrid_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v2__PropertyGrid__Row_msg_init;
extern const upb_MiniTable* grpc__channelz__v2__PropertyGrid__Row_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v2__PropertyTable_msg_init;
extern const upb_MiniTable* grpc__channelz__v2__PropertyTable_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v2__PropertyTable__Row_msg_init;
extern const upb_MiniTable* grpc__channelz__v2__PropertyTable__Row_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v2__PropertyValue_msg_init;
extern const upb_MiniTable* grpc__channelz__v2__PropertyValue_msg_init_ptr;

extern const upb_MiniTableFile src_proto_grpc_channelz_v2_property_list_proto_upb_file_layout;

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* SRC_PROTO_GRPC_CHANNELZ_V2_PROPERTY_LIST_PROTO_UPB_H__UPB_MINITABLE_H_ */
