/* This file was generated by upb_generator from the input file:
 *
 *     src/proto/grpc/channelz/channelz.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#ifndef SRC_PROTO_GRPC_CHANNELZ_CHANNELZ_PROTO_UPB_H__UPB_MINITABLE_H_
#define SRC_PROTO_GRPC_CHANNELZ_CHANNELZ_PROTO_UPB_H__UPB_MINITABLE_H_

#include "upb/generated_code_support.h"

// Must be last.
#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern const upb_MiniTable grpc__channelz__v1__Channel_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__Channel_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__Subchannel_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__Subchannel_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__ChannelConnectivityState_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__ChannelConnectivityState_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__ChannelData_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__ChannelData_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__ChannelTraceEvent_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__ChannelTraceEvent_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__ChannelTrace_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__ChannelTrace_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__ChannelRef_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__ChannelRef_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__SubchannelRef_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__SubchannelRef_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__SocketRef_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__SocketRef_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__ServerRef_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__ServerRef_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__Server_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__Server_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__ServerData_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__ServerData_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__Socket_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__Socket_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__SocketData_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__SocketData_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__Address_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__Address_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__Address__TcpIpAddress_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__Address__TcpIpAddress_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__Address__UdsAddress_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__Address__UdsAddress_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__Address__OtherAddress_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__Address__OtherAddress_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__Security_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__Security_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__Security__Tls_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__Security__Tls_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__Security__OtherSecurity_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__Security__OtherSecurity_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__SocketOption_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__SocketOption_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__SocketOptionTimeout_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__SocketOptionTimeout_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__SocketOptionLinger_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__SocketOptionLinger_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__SocketOptionTcpInfo_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__SocketOptionTcpInfo_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__GetTopChannelsRequest_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__GetTopChannelsRequest_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__GetTopChannelsResponse_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__GetTopChannelsResponse_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__GetServersRequest_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__GetServersRequest_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__GetServersResponse_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__GetServersResponse_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__GetServerRequest_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__GetServerRequest_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__GetServerResponse_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__GetServerResponse_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__GetServerSocketsRequest_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__GetServerSocketsRequest_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__GetServerSocketsResponse_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__GetServerSocketsResponse_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__GetChannelRequest_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__GetChannelRequest_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__GetChannelResponse_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__GetChannelResponse_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__GetSubchannelRequest_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__GetSubchannelRequest_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__GetSubchannelResponse_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__GetSubchannelResponse_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__GetSocketRequest_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__GetSocketRequest_msg_init_ptr;
extern const upb_MiniTable grpc__channelz__v1__GetSocketResponse_msg_init;
extern const upb_MiniTable* grpc__channelz__v1__GetSocketResponse_msg_init_ptr;

extern const upb_MiniTableFile src_proto_grpc_channelz_channelz_proto_upb_file_layout;

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* SRC_PROTO_GRPC_CHANNELZ_CHANNELZ_PROTO_UPB_H__UPB_MINITABLE_H_ */
