/* This file was generated by upb_generator from the input file:
 *
 *     src/proto/grpc/channelz/channelz.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#include <stddef.h>
#include "upb/generated_code_support.h"
#include "src/proto/grpc/channelz/channelz.upb_minitable.h"
#include "google/protobuf/any.upb_minitable.h"
#include "google/protobuf/duration.upb_minitable.h"
#include "google/protobuf/timestamp.upb_minitable.h"
#include "google/protobuf/wrappers.upb_minitable.h"

// Must be last.
#include "upb/port/def.inc"

extern const struct upb_MiniTable UPB_PRIVATE(_kUpb_MiniTable_StaticallyTreeShaken);
static const upb_MiniTableSubInternal grpc_channelz_v1_Channel__submsgs[5] = {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__ChannelRef_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__ChannelData_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__ChannelRef_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__SubchannelRef_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__SocketRef_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v1_Channel__fields[5] = {
  {1, UPB_SIZE(12, 16), 64, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(16, 24), 65, 1, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(20, 32), 0, 2, 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(24, 40), 0, 3, 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {5, UPB_SIZE(28, 48), 0, 4, 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__Channel_msg_init = {
  &grpc_channelz_v1_Channel__submsgs[0],
  &grpc_channelz_v1_Channel__fields[0],
  UPB_SIZE(32, 56), 5, kUpb_ExtMode_NonExtendable, 5, UPB_FASTTABLE_MASK(56), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.Channel",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x002000003f02001a, &upb_prm_1bt_max64b},
    {0x002800003f030022, &upb_prm_1bt_max64b},
    {0x003000003f04002a, &upb_prm_1bt_max64b},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

const upb_MiniTable* grpc__channelz__v1__Channel_msg_init_ptr = &grpc__channelz__v1__Channel_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v1_Subchannel__submsgs[5] = {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__SubchannelRef_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__ChannelData_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__ChannelRef_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__SubchannelRef_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__SocketRef_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v1_Subchannel__fields[5] = {
  {1, UPB_SIZE(12, 16), 64, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(16, 24), 65, 1, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(20, 32), 0, 2, 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(24, 40), 0, 3, 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {5, UPB_SIZE(28, 48), 0, 4, 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__Subchannel_msg_init = {
  &grpc_channelz_v1_Subchannel__submsgs[0],
  &grpc_channelz_v1_Subchannel__fields[0],
  UPB_SIZE(32, 56), 5, kUpb_ExtMode_NonExtendable, 5, UPB_FASTTABLE_MASK(56), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.Subchannel",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x002000003f02001a, &upb_prm_1bt_max64b},
    {0x002800003f030022, &upb_prm_1bt_max64b},
    {0x003000003f04002a, &upb_prm_1bt_max64b},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

const upb_MiniTable* grpc__channelz__v1__Subchannel_msg_init_ptr = &grpc__channelz__v1__Subchannel_msg_init;
static const upb_MiniTableField grpc_channelz_v1_ChannelConnectivityState__fields[1] = {
  {1, 8, 0, kUpb_NoSub, 5, (int)kUpb_FieldMode_Scalar | (int)kUpb_LabelFlags_IsAlternate | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__ChannelConnectivityState_msg_init = {
  NULL,
  &grpc_channelz_v1_ChannelConnectivityState__fields[0],
  16, 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.ChannelConnectivityState",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f000008, &upb_psv4_1bt},
  })
};

const upb_MiniTable* grpc__channelz__v1__ChannelConnectivityState_msg_init_ptr = &grpc__channelz__v1__ChannelConnectivityState_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v1_ChannelData__submsgs[3] = {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__ChannelConnectivityState_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__ChannelTrace_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &google__protobuf__Timestamp_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v1_ChannelData__fields[7] = {
  {1, UPB_SIZE(12, 32), 64, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(24, 16), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(16, 40), 65, 1, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(32, 48), 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {5, UPB_SIZE(40, 56), 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {6, UPB_SIZE(48, 64), 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {7, UPB_SIZE(20, 72), 66, 2, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__ChannelData_msg_init = {
  &grpc_channelz_v1_ChannelData__submsgs[0],
  &grpc_channelz_v1_ChannelData__fields[0],
  UPB_SIZE(56, 80), 7, kUpb_ExtMode_NonExtendable, 7, UPB_FASTTABLE_MASK(56), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.ChannelData",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f000012, &upb_pss_1bt},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x003000003f000020, &upb_psv8_1bt},
    {0x003800003f000028, &upb_psv8_1bt},
    {0x004000003f000030, &upb_psv8_1bt},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

const upb_MiniTable* grpc__channelz__v1__ChannelData_msg_init_ptr = &grpc__channelz__v1__ChannelData_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v1_ChannelTraceEvent__submsgs[3] = {
  {.UPB_PRIVATE(submsg) = &google__protobuf__Timestamp_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__ChannelRef_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__SubchannelRef_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v1_ChannelTraceEvent__fields[5] = {
  {1, UPB_SIZE(28, 24), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, 12, 0, kUpb_NoSub, 5, (int)kUpb_FieldMode_Scalar | (int)kUpb_LabelFlags_IsAlternate | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(16, 40), 64, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(24, 48), UPB_SIZE(-21, -17), 1, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {5, UPB_SIZE(24, 48), UPB_SIZE(-21, -17), 2, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__ChannelTraceEvent_msg_init = {
  &grpc_channelz_v1_ChannelTraceEvent__submsgs[0],
  &grpc_channelz_v1_ChannelTraceEvent__fields[0],
  UPB_SIZE(40, 56), 5, kUpb_ExtMode_NonExtendable, 5, UPB_FASTTABLE_MASK(56), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.ChannelTraceEvent",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001800003f00000a, &upb_pss_1bt},
    {0x000c00003f000010, &upb_psv4_1bt},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0030001004010022, &upb_pom_1bt_max64b},
    {0x003000100502002a, &upb_pom_1bt_max64b},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

const upb_MiniTable* grpc__channelz__v1__ChannelTraceEvent_msg_init_ptr = &grpc__channelz__v1__ChannelTraceEvent_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v1_ChannelTrace__submsgs[2] = {
  {.UPB_PRIVATE(submsg) = &google__protobuf__Timestamp_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__ChannelTraceEvent_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v1_ChannelTrace__fields[3] = {
  {1, UPB_SIZE(24, 16), 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 24), 64, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(16, 32), 0, 1, 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__ChannelTrace_msg_init = {
  &grpc_channelz_v1_ChannelTrace__submsgs[0],
  &grpc_channelz_v1_ChannelTrace__fields[0],
  UPB_SIZE(32, 40), 3, kUpb_ExtMode_NonExtendable, 3, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.ChannelTrace",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f000008, &upb_psv8_1bt},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x002000003f01001a, &upb_prm_1bt_max64b},
  })
};

const upb_MiniTable* grpc__channelz__v1__ChannelTrace_msg_init_ptr = &grpc__channelz__v1__ChannelTrace_msg_init;
static const upb_MiniTableField grpc_channelz_v1_ChannelRef__fields[2] = {
  {1, UPB_SIZE(16, 24), 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {2, 8, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__ChannelRef_msg_init = {
  NULL,
  &grpc_channelz_v1_ChannelRef__fields[0],
  UPB_SIZE(24, 32), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.ChannelRef",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001800003f000008, &upb_psv8_1bt},
    {0x000800003f000012, &upb_pss_1bt},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

const upb_MiniTable* grpc__channelz__v1__ChannelRef_msg_init_ptr = &grpc__channelz__v1__ChannelRef_msg_init;
static const upb_MiniTableField grpc_channelz_v1_SubchannelRef__fields[2] = {
  {7, UPB_SIZE(16, 24), 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {8, 8, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__SubchannelRef_msg_init = {
  NULL,
  &grpc_channelz_v1_SubchannelRef__fields[0],
  UPB_SIZE(24, 32), 2, kUpb_ExtMode_NonExtendable, 0, UPB_FASTTABLE_MASK(120), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.SubchannelRef",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001800003f000038, &upb_psv8_1bt},
    {0x000800003f000042, &upb_pss_1bt},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

const upb_MiniTable* grpc__channelz__v1__SubchannelRef_msg_init_ptr = &grpc__channelz__v1__SubchannelRef_msg_init;
static const upb_MiniTableField grpc_channelz_v1_SocketRef__fields[2] = {
  {3, UPB_SIZE(16, 24), 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {4, 8, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__SocketRef_msg_init = {
  NULL,
  &grpc_channelz_v1_SocketRef__fields[0],
  UPB_SIZE(24, 32), 2, kUpb_ExtMode_NonExtendable, 0, UPB_FASTTABLE_MASK(56), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.SocketRef",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001800003f000018, &upb_psv8_1bt},
    {0x000800003f000022, &upb_pss_1bt},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

const upb_MiniTable* grpc__channelz__v1__SocketRef_msg_init_ptr = &grpc__channelz__v1__SocketRef_msg_init;
static const upb_MiniTableField grpc_channelz_v1_ServerRef__fields[2] = {
  {5, UPB_SIZE(16, 24), 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {6, 8, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__ServerRef_msg_init = {
  NULL,
  &grpc_channelz_v1_ServerRef__fields[0],
  UPB_SIZE(24, 32), 2, kUpb_ExtMode_NonExtendable, 0, UPB_FASTTABLE_MASK(56), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.ServerRef",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001800003f000028, &upb_psv8_1bt},
    {0x000800003f000032, &upb_pss_1bt},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

const upb_MiniTable* grpc__channelz__v1__ServerRef_msg_init_ptr = &grpc__channelz__v1__ServerRef_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v1_Server__submsgs[3] = {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__ServerRef_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__ServerData_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__SocketRef_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v1_Server__fields[3] = {
  {1, UPB_SIZE(12, 16), 64, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(16, 24), 65, 1, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(20, 32), 0, 2, 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__Server_msg_init = {
  &grpc_channelz_v1_Server__submsgs[0],
  &grpc_channelz_v1_Server__fields[0],
  UPB_SIZE(24, 40), 3, kUpb_ExtMode_NonExtendable, 3, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.Server",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x002000003f02001a, &upb_prm_1bt_max64b},
  })
};

const upb_MiniTable* grpc__channelz__v1__Server_msg_init_ptr = &grpc__channelz__v1__Server_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v1_ServerData__submsgs[2] = {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__ChannelTrace_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &google__protobuf__Timestamp_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v1_ServerData__fields[5] = {
  {1, UPB_SIZE(12, 16), 64, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, 24, 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {3, 32, 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {4, 40, 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {5, UPB_SIZE(16, 48), 65, 1, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__ServerData_msg_init = {
  &grpc_channelz_v1_ServerData__submsgs[0],
  &grpc_channelz_v1_ServerData__fields[0],
  UPB_SIZE(48, 56), 5, kUpb_ExtMode_NonExtendable, 5, UPB_FASTTABLE_MASK(56), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.ServerData",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001800003f000010, &upb_psv8_1bt},
    {0x002000003f000018, &upb_psv8_1bt},
    {0x002800003f000020, &upb_psv8_1bt},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

const upb_MiniTable* grpc__channelz__v1__ServerData_msg_init_ptr = &grpc__channelz__v1__ServerData_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v1_Socket__submsgs[5] = {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__SocketRef_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__SocketData_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__Address_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__Address_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__Security_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v1_Socket__fields[6] = {
  {1, UPB_SIZE(12, 32), 64, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(16, 40), 65, 1, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(20, 48), 66, 2, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(24, 56), 67, 3, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {5, UPB_SIZE(28, 64), 68, 4, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {6, UPB_SIZE(32, 16), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__Socket_msg_init = {
  &grpc_channelz_v1_Socket__submsgs[0],
  &grpc_channelz_v1_Socket__fields[0],
  UPB_SIZE(40, 72), 6, kUpb_ExtMode_NonExtendable, 6, UPB_FASTTABLE_MASK(56), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.Socket",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f000032, &upb_pss_1bt},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

const upb_MiniTable* grpc__channelz__v1__Socket_msg_init_ptr = &grpc__channelz__v1__Socket_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v1_SocketData__submsgs[7] = {
  {.UPB_PRIVATE(submsg) = &google__protobuf__Timestamp_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &google__protobuf__Timestamp_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &google__protobuf__Timestamp_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &google__protobuf__Timestamp_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &google__protobuf__Int64Value_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &google__protobuf__Int64Value_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__SocketOption_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v1_SocketData__fields[13] = {
  {1, UPB_SIZE(40, 16), 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(48, 24), 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(56, 32), 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(64, 40), 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {5, UPB_SIZE(72, 48), 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {6, UPB_SIZE(80, 56), 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {7, UPB_SIZE(12, 64), 64, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {8, UPB_SIZE(16, 72), 65, 1, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {9, UPB_SIZE(20, 80), 66, 2, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {10, UPB_SIZE(24, 88), 67, 3, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {11, UPB_SIZE(28, 96), 68, 4, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {12, UPB_SIZE(32, 104), 69, 5, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {13, UPB_SIZE(36, 112), 0, 6, 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__SocketData_msg_init = {
  &grpc_channelz_v1_SocketData__submsgs[0],
  &grpc_channelz_v1_SocketData__fields[0],
  UPB_SIZE(88, 120), 13, kUpb_ExtMode_NonExtendable, 13, UPB_FASTTABLE_MASK(120), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.SocketData",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f000008, &upb_psv8_1bt},
    {0x001800003f000010, &upb_psv8_1bt},
    {0x002000003f000018, &upb_psv8_1bt},
    {0x002800003f000020, &upb_psv8_1bt},
    {0x003000003f000028, &upb_psv8_1bt},
    {0x003800003f000030, &upb_psv8_1bt},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x007000003f06006a, &upb_prm_1bt_max64b},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

const upb_MiniTable* grpc__channelz__v1__SocketData_msg_init_ptr = &grpc__channelz__v1__SocketData_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v1_Address__submsgs[3] = {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__Address__TcpIpAddress_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__Address__UdsAddress_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__Address__OtherAddress_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v1_Address__fields[3] = {
  {1, UPB_SIZE(12, 16), -9, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 16), -9, 1, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(12, 16), -9, 2, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__Address_msg_init = {
  &grpc_channelz_v1_Address__submsgs[0],
  &grpc_channelz_v1_Address__fields[0],
  UPB_SIZE(16, 24), 3, kUpb_ExtMode_NonExtendable, 3, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.Address",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000080100000a, &upb_pom_1bt_max64b},
    {0x0010000802010012, &upb_pom_1bt_max64b},
    {0x001000080302001a, &upb_pom_1bt_max64b},
  })
};

const upb_MiniTable* grpc__channelz__v1__Address_msg_init_ptr = &grpc__channelz__v1__Address_msg_init;
static const upb_MiniTableField grpc_channelz_v1_Address_TcpIpAddress__fields[2] = {
  {1, UPB_SIZE(12, 16), 0, kUpb_NoSub, 12, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, 8, 0, kUpb_NoSub, 5, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__Address__TcpIpAddress_msg_init = {
  NULL,
  &grpc_channelz_v1_Address_TcpIpAddress__fields[0],
  UPB_SIZE(24, 32), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.Address.TcpIpAddress",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_psb_1bt},
    {0x000800003f000010, &upb_psv4_1bt},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

const upb_MiniTable* grpc__channelz__v1__Address__TcpIpAddress_msg_init_ptr = &grpc__channelz__v1__Address__TcpIpAddress_msg_init;
static const upb_MiniTableField grpc_channelz_v1_Address_UdsAddress__fields[1] = {
  {1, 8, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__Address__UdsAddress_msg_init = {
  NULL,
  &grpc_channelz_v1_Address_UdsAddress__fields[0],
  UPB_SIZE(16, 24), 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.Address.UdsAddress",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f00000a, &upb_pss_1bt},
  })
};

const upb_MiniTable* grpc__channelz__v1__Address__UdsAddress_msg_init_ptr = &grpc__channelz__v1__Address__UdsAddress_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v1_Address_OtherAddress__submsgs[1] = {
  {.UPB_PRIVATE(submsg) = &google__protobuf__Any_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v1_Address_OtherAddress__fields[2] = {
  {1, 16, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 32), 64, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__Address__OtherAddress_msg_init = {
  &grpc_channelz_v1_Address_OtherAddress__submsgs[0],
  &grpc_channelz_v1_Address_OtherAddress__fields[0],
  UPB_SIZE(24, 40), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.Address.OtherAddress",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_pss_1bt},
  })
};

const upb_MiniTable* grpc__channelz__v1__Address__OtherAddress_msg_init_ptr = &grpc__channelz__v1__Address__OtherAddress_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v1_Security__submsgs[2] = {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__Security__Tls_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__Security__OtherSecurity_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v1_Security__fields[2] = {
  {1, UPB_SIZE(12, 16), -9, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 16), -9, 1, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__Security_msg_init = {
  &grpc_channelz_v1_Security__submsgs[0],
  &grpc_channelz_v1_Security__fields[0],
  UPB_SIZE(16, 24), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.Security",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000080100000a, &upb_pom_1bt_max128b},
    {0x0010000802010012, &upb_pom_1bt_max64b},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

const upb_MiniTable* grpc__channelz__v1__Security_msg_init_ptr = &grpc__channelz__v1__Security_msg_init;
static const upb_MiniTableField grpc_channelz_v1_Security_Tls__fields[4] = {
  {1, UPB_SIZE(28, 48), -9, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(28, 48), -9, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(12, 16), 0, kUpb_NoSub, 12, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(20, 32), 0, kUpb_NoSub, 12, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__Security__Tls_msg_init = {
  NULL,
  &grpc_channelz_v1_Security_Tls__fields[0],
  UPB_SIZE(40, 64), 4, kUpb_ExtMode_NonExtendable, 4, UPB_FASTTABLE_MASK(56), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.Security.Tls",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x003000080100000a, &upb_pos_1bt},
    {0x0030000802000012, &upb_pos_1bt},
    {0x001000003f00001a, &upb_psb_1bt},
    {0x002000003f000022, &upb_psb_1bt},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

const upb_MiniTable* grpc__channelz__v1__Security__Tls_msg_init_ptr = &grpc__channelz__v1__Security__Tls_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v1_Security_OtherSecurity__submsgs[1] = {
  {.UPB_PRIVATE(submsg) = &google__protobuf__Any_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v1_Security_OtherSecurity__fields[2] = {
  {1, 16, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 32), 64, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__Security__OtherSecurity_msg_init = {
  &grpc_channelz_v1_Security_OtherSecurity__submsgs[0],
  &grpc_channelz_v1_Security_OtherSecurity__fields[0],
  UPB_SIZE(24, 40), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.Security.OtherSecurity",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_pss_1bt},
  })
};

const upb_MiniTable* grpc__channelz__v1__Security__OtherSecurity_msg_init_ptr = &grpc__channelz__v1__Security__OtherSecurity_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v1_SocketOption__submsgs[1] = {
  {.UPB_PRIVATE(submsg) = &google__protobuf__Any_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v1_SocketOption__fields[3] = {
  {1, 16, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(24, 32), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(12, 48), 64, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__SocketOption_msg_init = {
  &grpc_channelz_v1_SocketOption__submsgs[0],
  &grpc_channelz_v1_SocketOption__fields[0],
  UPB_SIZE(32, 56), 3, kUpb_ExtMode_NonExtendable, 3, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.SocketOption",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_pss_1bt},
    {0x002000003f000012, &upb_pss_1bt},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

const upb_MiniTable* grpc__channelz__v1__SocketOption_msg_init_ptr = &grpc__channelz__v1__SocketOption_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v1_SocketOptionTimeout__submsgs[1] = {
  {.UPB_PRIVATE(submsg) = &google__protobuf__Duration_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v1_SocketOptionTimeout__fields[1] = {
  {1, UPB_SIZE(12, 16), 64, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__SocketOptionTimeout_msg_init = {
  &grpc_channelz_v1_SocketOptionTimeout__submsgs[0],
  &grpc_channelz_v1_SocketOptionTimeout__fields[0],
  UPB_SIZE(16, 24), 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.SocketOptionTimeout",
#endif
};

const upb_MiniTable* grpc__channelz__v1__SocketOptionTimeout_msg_init_ptr = &grpc__channelz__v1__SocketOptionTimeout_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v1_SocketOptionLinger__submsgs[1] = {
  {.UPB_PRIVATE(submsg) = &google__protobuf__Duration_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v1_SocketOptionLinger__fields[2] = {
  {1, 9, 0, kUpb_NoSub, 8, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_1Byte << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 16), 64, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__SocketOptionLinger_msg_init = {
  &grpc_channelz_v1_SocketOptionLinger__submsgs[0],
  &grpc_channelz_v1_SocketOptionLinger__fields[0],
  UPB_SIZE(16, 24), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.SocketOptionLinger",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000900003f000008, &upb_psb1_1bt},
  })
};

const upb_MiniTable* grpc__channelz__v1__SocketOptionLinger_msg_init_ptr = &grpc__channelz__v1__SocketOptionLinger_msg_init;
static const upb_MiniTableField grpc_channelz_v1_SocketOptionTcpInfo__fields[29] = {
  {1, 8, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {2, 12, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {3, 16, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {4, 20, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {5, 24, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {6, 28, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {7, 32, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {8, 36, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {9, 40, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {10, 44, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {11, 48, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {12, 52, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {13, 56, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {14, 60, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {15, 64, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {16, 68, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {17, 72, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {18, 76, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {19, 80, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {20, 84, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {21, 88, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {22, 92, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {23, 96, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {24, 100, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {25, 104, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {26, 108, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {27, 112, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {28, 116, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {29, 120, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__SocketOptionTcpInfo_msg_init = {
  NULL,
  &grpc_channelz_v1_SocketOptionTcpInfo__fields[0],
  128, 29, kUpb_ExtMode_NonExtendable, 29, UPB_FASTTABLE_MASK(248), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.SocketOptionTcpInfo",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f000008, &upb_psv4_1bt},
    {0x000c00003f000010, &upb_psv4_1bt},
    {0x001000003f000018, &upb_psv4_1bt},
    {0x001400003f000020, &upb_psv4_1bt},
    {0x001800003f000028, &upb_psv4_1bt},
    {0x001c00003f000030, &upb_psv4_1bt},
    {0x002000003f000038, &upb_psv4_1bt},
    {0x002400003f000040, &upb_psv4_1bt},
    {0x002800003f000048, &upb_psv4_1bt},
    {0x002c00003f000050, &upb_psv4_1bt},
    {0x003000003f000058, &upb_psv4_1bt},
    {0x003400003f000060, &upb_psv4_1bt},
    {0x003800003f000068, &upb_psv4_1bt},
    {0x003c00003f000070, &upb_psv4_1bt},
    {0x004000003f000078, &upb_psv4_1bt},
    {0x004400003f000180, &upb_psv4_2bt},
    {0x004800003f000188, &upb_psv4_2bt},
    {0x004c00003f000190, &upb_psv4_2bt},
    {0x005000003f000198, &upb_psv4_2bt},
    {0x005400003f0001a0, &upb_psv4_2bt},
    {0x005800003f0001a8, &upb_psv4_2bt},
    {0x005c00003f0001b0, &upb_psv4_2bt},
    {0x006000003f0001b8, &upb_psv4_2bt},
    {0x006400003f0001c0, &upb_psv4_2bt},
    {0x006800003f0001c8, &upb_psv4_2bt},
    {0x006c00003f0001d0, &upb_psv4_2bt},
    {0x007000003f0001d8, &upb_psv4_2bt},
    {0x007400003f0001e0, &upb_psv4_2bt},
    {0x007800003f0001e8, &upb_psv4_2bt},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

const upb_MiniTable* grpc__channelz__v1__SocketOptionTcpInfo_msg_init_ptr = &grpc__channelz__v1__SocketOptionTcpInfo_msg_init;
static const upb_MiniTableField grpc_channelz_v1_GetTopChannelsRequest__fields[2] = {
  {1, 8, 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {2, 16, 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__GetTopChannelsRequest_msg_init = {
  NULL,
  &grpc_channelz_v1_GetTopChannelsRequest__fields[0],
  24, 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.GetTopChannelsRequest",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f000008, &upb_psv8_1bt},
    {0x001000003f000010, &upb_psv8_1bt},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

const upb_MiniTable* grpc__channelz__v1__GetTopChannelsRequest_msg_init_ptr = &grpc__channelz__v1__GetTopChannelsRequest_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v1_GetTopChannelsResponse__submsgs[1] = {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__Channel_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v1_GetTopChannelsResponse__fields[2] = {
  {1, UPB_SIZE(12, 16), 0, 0, 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, 8, 0, kUpb_NoSub, 8, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_1Byte << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__GetTopChannelsResponse_msg_init = {
  &grpc_channelz_v1_GetTopChannelsResponse__submsgs[0],
  &grpc_channelz_v1_GetTopChannelsResponse__fields[0],
  UPB_SIZE(16, 24), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.GetTopChannelsResponse",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_prm_1bt_max64b},
    {0x000800003f000010, &upb_psb1_1bt},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

const upb_MiniTable* grpc__channelz__v1__GetTopChannelsResponse_msg_init_ptr = &grpc__channelz__v1__GetTopChannelsResponse_msg_init;
static const upb_MiniTableField grpc_channelz_v1_GetServersRequest__fields[2] = {
  {1, 8, 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {2, 16, 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__GetServersRequest_msg_init = {
  NULL,
  &grpc_channelz_v1_GetServersRequest__fields[0],
  24, 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.GetServersRequest",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f000008, &upb_psv8_1bt},
    {0x001000003f000010, &upb_psv8_1bt},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

const upb_MiniTable* grpc__channelz__v1__GetServersRequest_msg_init_ptr = &grpc__channelz__v1__GetServersRequest_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v1_GetServersResponse__submsgs[1] = {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__Server_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v1_GetServersResponse__fields[2] = {
  {1, UPB_SIZE(12, 16), 0, 0, 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, 8, 0, kUpb_NoSub, 8, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_1Byte << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__GetServersResponse_msg_init = {
  &grpc_channelz_v1_GetServersResponse__submsgs[0],
  &grpc_channelz_v1_GetServersResponse__fields[0],
  UPB_SIZE(16, 24), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.GetServersResponse",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_prm_1bt_max64b},
    {0x000800003f000010, &upb_psb1_1bt},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

const upb_MiniTable* grpc__channelz__v1__GetServersResponse_msg_init_ptr = &grpc__channelz__v1__GetServersResponse_msg_init;
static const upb_MiniTableField grpc_channelz_v1_GetServerRequest__fields[1] = {
  {1, 8, 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__GetServerRequest_msg_init = {
  NULL,
  &grpc_channelz_v1_GetServerRequest__fields[0],
  16, 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.GetServerRequest",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f000008, &upb_psv8_1bt},
  })
};

const upb_MiniTable* grpc__channelz__v1__GetServerRequest_msg_init_ptr = &grpc__channelz__v1__GetServerRequest_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v1_GetServerResponse__submsgs[1] = {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__Server_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v1_GetServerResponse__fields[1] = {
  {1, UPB_SIZE(12, 16), 64, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__GetServerResponse_msg_init = {
  &grpc_channelz_v1_GetServerResponse__submsgs[0],
  &grpc_channelz_v1_GetServerResponse__fields[0],
  UPB_SIZE(16, 24), 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.GetServerResponse",
#endif
};

const upb_MiniTable* grpc__channelz__v1__GetServerResponse_msg_init_ptr = &grpc__channelz__v1__GetServerResponse_msg_init;
static const upb_MiniTableField grpc_channelz_v1_GetServerSocketsRequest__fields[3] = {
  {1, 8, 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {2, 16, 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {3, 24, 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__GetServerSocketsRequest_msg_init = {
  NULL,
  &grpc_channelz_v1_GetServerSocketsRequest__fields[0],
  32, 3, kUpb_ExtMode_NonExtendable, 3, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.GetServerSocketsRequest",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f000008, &upb_psv8_1bt},
    {0x001000003f000010, &upb_psv8_1bt},
    {0x001800003f000018, &upb_psv8_1bt},
  })
};

const upb_MiniTable* grpc__channelz__v1__GetServerSocketsRequest_msg_init_ptr = &grpc__channelz__v1__GetServerSocketsRequest_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v1_GetServerSocketsResponse__submsgs[1] = {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__SocketRef_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v1_GetServerSocketsResponse__fields[2] = {
  {1, UPB_SIZE(12, 16), 0, 0, 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, 8, 0, kUpb_NoSub, 8, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_1Byte << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__GetServerSocketsResponse_msg_init = {
  &grpc_channelz_v1_GetServerSocketsResponse__submsgs[0],
  &grpc_channelz_v1_GetServerSocketsResponse__fields[0],
  UPB_SIZE(16, 24), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.GetServerSocketsResponse",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_prm_1bt_max64b},
    {0x000800003f000010, &upb_psb1_1bt},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

const upb_MiniTable* grpc__channelz__v1__GetServerSocketsResponse_msg_init_ptr = &grpc__channelz__v1__GetServerSocketsResponse_msg_init;
static const upb_MiniTableField grpc_channelz_v1_GetChannelRequest__fields[1] = {
  {1, 8, 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__GetChannelRequest_msg_init = {
  NULL,
  &grpc_channelz_v1_GetChannelRequest__fields[0],
  16, 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.GetChannelRequest",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f000008, &upb_psv8_1bt},
  })
};

const upb_MiniTable* grpc__channelz__v1__GetChannelRequest_msg_init_ptr = &grpc__channelz__v1__GetChannelRequest_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v1_GetChannelResponse__submsgs[1] = {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__Channel_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v1_GetChannelResponse__fields[1] = {
  {1, UPB_SIZE(12, 16), 64, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__GetChannelResponse_msg_init = {
  &grpc_channelz_v1_GetChannelResponse__submsgs[0],
  &grpc_channelz_v1_GetChannelResponse__fields[0],
  UPB_SIZE(16, 24), 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.GetChannelResponse",
#endif
};

const upb_MiniTable* grpc__channelz__v1__GetChannelResponse_msg_init_ptr = &grpc__channelz__v1__GetChannelResponse_msg_init;
static const upb_MiniTableField grpc_channelz_v1_GetSubchannelRequest__fields[1] = {
  {1, 8, 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__GetSubchannelRequest_msg_init = {
  NULL,
  &grpc_channelz_v1_GetSubchannelRequest__fields[0],
  16, 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.GetSubchannelRequest",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f000008, &upb_psv8_1bt},
  })
};

const upb_MiniTable* grpc__channelz__v1__GetSubchannelRequest_msg_init_ptr = &grpc__channelz__v1__GetSubchannelRequest_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v1_GetSubchannelResponse__submsgs[1] = {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__Subchannel_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v1_GetSubchannelResponse__fields[1] = {
  {1, UPB_SIZE(12, 16), 64, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__GetSubchannelResponse_msg_init = {
  &grpc_channelz_v1_GetSubchannelResponse__submsgs[0],
  &grpc_channelz_v1_GetSubchannelResponse__fields[0],
  UPB_SIZE(16, 24), 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.GetSubchannelResponse",
#endif
};

const upb_MiniTable* grpc__channelz__v1__GetSubchannelResponse_msg_init_ptr = &grpc__channelz__v1__GetSubchannelResponse_msg_init;
static const upb_MiniTableField grpc_channelz_v1_GetSocketRequest__fields[2] = {
  {1, 16, 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {2, 8, 0, kUpb_NoSub, 8, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_1Byte << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__GetSocketRequest_msg_init = {
  NULL,
  &grpc_channelz_v1_GetSocketRequest__fields[0],
  24, 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.GetSocketRequest",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f000008, &upb_psv8_1bt},
    {0x000800003f000010, &upb_psb1_1bt},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

const upb_MiniTable* grpc__channelz__v1__GetSocketRequest_msg_init_ptr = &grpc__channelz__v1__GetSocketRequest_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v1_GetSocketResponse__submsgs[1] = {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v1__Socket_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v1_GetSocketResponse__fields[1] = {
  {1, UPB_SIZE(12, 16), 64, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v1__GetSocketResponse_msg_init = {
  &grpc_channelz_v1_GetSocketResponse__submsgs[0],
  &grpc_channelz_v1_GetSocketResponse__fields[0],
  UPB_SIZE(16, 24), 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v1.GetSocketResponse",
#endif
};

const upb_MiniTable* grpc__channelz__v1__GetSocketResponse_msg_init_ptr = &grpc__channelz__v1__GetSocketResponse_msg_init;
static const upb_MiniTable *messages_layout[39] = {
  &grpc__channelz__v1__Channel_msg_init,
  &grpc__channelz__v1__Subchannel_msg_init,
  &grpc__channelz__v1__ChannelConnectivityState_msg_init,
  &grpc__channelz__v1__ChannelData_msg_init,
  &grpc__channelz__v1__ChannelTraceEvent_msg_init,
  &grpc__channelz__v1__ChannelTrace_msg_init,
  &grpc__channelz__v1__ChannelRef_msg_init,
  &grpc__channelz__v1__SubchannelRef_msg_init,
  &grpc__channelz__v1__SocketRef_msg_init,
  &grpc__channelz__v1__ServerRef_msg_init,
  &grpc__channelz__v1__Server_msg_init,
  &grpc__channelz__v1__ServerData_msg_init,
  &grpc__channelz__v1__Socket_msg_init,
  &grpc__channelz__v1__SocketData_msg_init,
  &grpc__channelz__v1__Address_msg_init,
  &grpc__channelz__v1__Address__TcpIpAddress_msg_init,
  &grpc__channelz__v1__Address__UdsAddress_msg_init,
  &grpc__channelz__v1__Address__OtherAddress_msg_init,
  &grpc__channelz__v1__Security_msg_init,
  &grpc__channelz__v1__Security__Tls_msg_init,
  &grpc__channelz__v1__Security__OtherSecurity_msg_init,
  &grpc__channelz__v1__SocketOption_msg_init,
  &grpc__channelz__v1__SocketOptionTimeout_msg_init,
  &grpc__channelz__v1__SocketOptionLinger_msg_init,
  &grpc__channelz__v1__SocketOptionTcpInfo_msg_init,
  &grpc__channelz__v1__GetTopChannelsRequest_msg_init,
  &grpc__channelz__v1__GetTopChannelsResponse_msg_init,
  &grpc__channelz__v1__GetServersRequest_msg_init,
  &grpc__channelz__v1__GetServersResponse_msg_init,
  &grpc__channelz__v1__GetServerRequest_msg_init,
  &grpc__channelz__v1__GetServerResponse_msg_init,
  &grpc__channelz__v1__GetServerSocketsRequest_msg_init,
  &grpc__channelz__v1__GetServerSocketsResponse_msg_init,
  &grpc__channelz__v1__GetChannelRequest_msg_init,
  &grpc__channelz__v1__GetChannelResponse_msg_init,
  &grpc__channelz__v1__GetSubchannelRequest_msg_init,
  &grpc__channelz__v1__GetSubchannelResponse_msg_init,
  &grpc__channelz__v1__GetSocketRequest_msg_init,
  &grpc__channelz__v1__GetSocketResponse_msg_init,
};

const upb_MiniTableFile src_proto_grpc_channelz_channelz_proto_upb_file_layout = {
  messages_layout,
  NULL,
  NULL,
  39,
  0,
  0,
};

#include "upb/port/undef.inc"

