/* This file was generated by upb_generator from the input file:
 *
 *     src/proto/grpc/channelz/v2/promise.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#include <stddef.h>
#include "upb/generated_code_support.h"
#include "src/proto/grpc/channelz/v2/promise.upb_minitable.h"
#include "src/proto/grpc/channelz/v2/property_list.upb_minitable.h"

// Must be last.
#include "upb/port/def.inc"

extern const struct upb_MiniTable UPB_PRIVATE(_kUpb_MiniTable_StaticallyTreeShaken);
static const upb_MiniTableSubInternal grpc_channelz_v2_Promise__submsgs[7] = {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v2__Promise__Custom_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v2__Promise__If_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v2__Promise__Loop_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v2__Promise__Race_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v2__Promise__Seq_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v2__Promise__Join_msg_init_ptr},
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v2__Promise__Map_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v2_Promise__fields[8] = {
  {1, UPB_SIZE(12, 16), -9, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 16), -9, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(12, 16), -9, 1, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(12, 16), -9, 2, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {5, UPB_SIZE(12, 16), -9, 3, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {6, UPB_SIZE(12, 16), -9, 4, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {7, UPB_SIZE(12, 16), -9, 5, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {8, UPB_SIZE(12, 16), -9, 6, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v2__Promise_msg_init = {
  &grpc_channelz_v2_Promise__submsgs[0],
  &grpc_channelz_v2_Promise__fields[0],
  UPB_SIZE(24, 32), 8, kUpb_ExtMode_NonExtendable, 8, UPB_FASTTABLE_MASK(120), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v2.Promise",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000080100000a, &upb_pom_1bt_max64b},
    {0x0010000802000012, &upb_pos_1bt},
    {0x001000080301001a, &upb_pom_1bt_max64b},
    {0x0010000804020022, &upb_pom_1bt_max64b},
    {0x001000080503002a, &upb_pom_1bt_max64b},
    {0x0010000806040032, &upb_pom_1bt_max64b},
    {0x001000080705003a, &upb_pom_1bt_max64b},
    {0x0010000808060042, &upb_pom_1bt_max64b},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

const upb_MiniTable* grpc__channelz__v2__Promise_msg_init_ptr = &grpc__channelz__v2__Promise_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v2_Promise_Custom__submsgs[1] = {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v2__PropertyList_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v2_Promise_Custom__fields[2] = {
  {1, 16, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 32), 64, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v2__Promise__Custom_msg_init = {
  &grpc_channelz_v2_Promise_Custom__submsgs[0],
  &grpc_channelz_v2_Promise_Custom__fields[0],
  UPB_SIZE(24, 40), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v2.Promise.Custom",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_pss_1bt},
  })
};

const upb_MiniTable* grpc__channelz__v2__Promise__Custom_msg_init_ptr = &grpc__channelz__v2__Promise__Custom_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v2_Promise_If__submsgs[1] = {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v2__Promise_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v2_Promise_If__fields[4] = {
  {1, 9, 0, kUpb_NoSub, 8, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_1Byte << kUpb_FieldRep_Shift)},
  {2, 16, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(24, 32), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(12, 48), 64, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v2__Promise__If_msg_init = {
  &grpc_channelz_v2_Promise_If__submsgs[0],
  &grpc_channelz_v2_Promise_If__fields[0],
  UPB_SIZE(32, 56), 4, kUpb_ExtMode_NonExtendable, 4, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v2.Promise.If",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000900003f000008, &upb_psb1_1bt},
    {0x001000003f000012, &upb_pss_1bt},
    {0x002000003f00001a, &upb_pss_1bt},
  })
};

const upb_MiniTable* grpc__channelz__v2__Promise__If_msg_init_ptr = &grpc__channelz__v2__Promise__If_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v2_Promise_Loop__submsgs[1] = {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v2__Promise_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v2_Promise_Loop__fields[3] = {
  {1, 16, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 32), 64, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, 9, 0, kUpb_NoSub, 8, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_1Byte << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v2__Promise__Loop_msg_init = {
  &grpc_channelz_v2_Promise_Loop__submsgs[0],
  &grpc_channelz_v2_Promise_Loop__fields[0],
  UPB_SIZE(24, 40), 3, kUpb_ExtMode_NonExtendable, 3, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v2.Promise.Loop",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_pss_1bt},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000900003f000018, &upb_psb1_1bt},
  })
};

const upb_MiniTable* grpc__channelz__v2__Promise__Loop_msg_init_ptr = &grpc__channelz__v2__Promise__Loop_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v2_Promise_Race__submsgs[1] = {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v2__Promise_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v2_Promise_Race__fields[1] = {
  {1, 8, 0, 0, 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v2__Promise__Race_msg_init = {
  &grpc_channelz_v2_Promise_Race__submsgs[0],
  &grpc_channelz_v2_Promise_Race__fields[0],
  16, 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v2.Promise.Race",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f00000a, &upb_prm_1bt_max64b},
  })
};

const upb_MiniTable* grpc__channelz__v2__Promise__Race_msg_init_ptr = &grpc__channelz__v2__Promise__Race_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v2_Promise_SeqStep__submsgs[1] = {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v2__Promise_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v2_Promise_SeqStep__fields[2] = {
  {1, 16, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 32), 64, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v2__Promise__SeqStep_msg_init = {
  &grpc_channelz_v2_Promise_SeqStep__submsgs[0],
  &grpc_channelz_v2_Promise_SeqStep__fields[0],
  UPB_SIZE(24, 40), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v2.Promise.SeqStep",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_pss_1bt},
  })
};

const upb_MiniTable* grpc__channelz__v2__Promise__SeqStep_msg_init_ptr = &grpc__channelz__v2__Promise__SeqStep_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v2_Promise_Seq__submsgs[1] = {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v2__Promise__SeqStep_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v2_Promise_Seq__fields[2] = {
  {1, 8, 0, kUpb_NoSub, 5, (int)kUpb_FieldMode_Scalar | (int)kUpb_LabelFlags_IsAlternate | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 16), 0, 0, 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v2__Promise__Seq_msg_init = {
  &grpc_channelz_v2_Promise_Seq__submsgs[0],
  &grpc_channelz_v2_Promise_Seq__fields[0],
  UPB_SIZE(16, 24), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v2.Promise.Seq",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f000008, &upb_psv4_1bt},
    {0x001000003f000012, &upb_prm_1bt_max64b},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

const upb_MiniTable* grpc__channelz__v2__Promise__Seq_msg_init_ptr = &grpc__channelz__v2__Promise__Seq_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v2_Promise_JoinBranch__submsgs[1] = {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v2__Promise_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v2_Promise_JoinBranch__fields[3] = {
  {1, UPB_SIZE(12, 16), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(20, 32), -9, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(20, 32), -9, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v2__Promise__JoinBranch_msg_init = {
  &grpc_channelz_v2_Promise_JoinBranch__submsgs[0],
  &grpc_channelz_v2_Promise_JoinBranch__fields[0],
  UPB_SIZE(32, 48), 3, kUpb_ExtMode_NonExtendable, 3, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v2.Promise.JoinBranch",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_pss_1bt},
    {0x0020000802000012, &upb_pom_1bt_max64b},
    {0x002000080300001a, &upb_pos_1bt},
  })
};

const upb_MiniTable* grpc__channelz__v2__Promise__JoinBranch_msg_init_ptr = &grpc__channelz__v2__Promise__JoinBranch_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v2_Promise_Join__submsgs[1] = {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v2__Promise__JoinBranch_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v2_Promise_Join__fields[2] = {
  {1, 8, 0, kUpb_NoSub, 5, (int)kUpb_FieldMode_Scalar | (int)kUpb_LabelFlags_IsAlternate | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 16), 0, 0, 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v2__Promise__Join_msg_init = {
  &grpc_channelz_v2_Promise_Join__submsgs[0],
  &grpc_channelz_v2_Promise_Join__fields[0],
  UPB_SIZE(16, 24), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v2.Promise.Join",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f000008, &upb_psv4_1bt},
    {0x001000003f000012, &upb_prm_1bt_max64b},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

const upb_MiniTable* grpc__channelz__v2__Promise__Join_msg_init_ptr = &grpc__channelz__v2__Promise__Join_msg_init;
static const upb_MiniTableSubInternal grpc_channelz_v2_Promise_Map__submsgs[1] = {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v2__Promise_msg_init_ptr},
};

static const upb_MiniTableField grpc_channelz_v2_Promise_Map__fields[2] = {
  {1, UPB_SIZE(12, 32), 64, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, 16, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc__channelz__v2__Promise__Map_msg_init = {
  &grpc_channelz_v2_Promise_Map__submsgs[0],
  &grpc_channelz_v2_Promise_Map__fields[0],
  UPB_SIZE(24, 40), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v2.Promise.Map",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f000012, &upb_pss_1bt},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

const upb_MiniTable* grpc__channelz__v2__Promise__Map_msg_init_ptr = &grpc__channelz__v2__Promise__Map_msg_init;
static const upb_MiniTable *messages_layout[10] = {
  &grpc__channelz__v2__Promise_msg_init,
  &grpc__channelz__v2__Promise__Custom_msg_init,
  &grpc__channelz__v2__Promise__If_msg_init,
  &grpc__channelz__v2__Promise__Loop_msg_init,
  &grpc__channelz__v2__Promise__Race_msg_init,
  &grpc__channelz__v2__Promise__SeqStep_msg_init,
  &grpc__channelz__v2__Promise__Seq_msg_init,
  &grpc__channelz__v2__Promise__JoinBranch_msg_init,
  &grpc__channelz__v2__Promise__Join_msg_init,
  &grpc__channelz__v2__Promise__Map_msg_init,
};

const upb_MiniTableFile src_proto_grpc_channelz_v2_promise_proto_upb_file_layout = {
  messages_layout,
  NULL,
  NULL,
  10,
  0,
  0,
};

#include "upb/port/undef.inc"

